Linear regression in high dimension and/or for correlated inputs
----------------------------------------------------------------

# ---------------------------------------------------
# Example of use of ridge regression (2 correlated inputs)
# ---------------------------------------------------
library(MASS)

b0=NULL;b1=NULL;b2=NULL;
for (j in 1 :50){
x1 = rnorm(20)
x2 = rnorm(20,mean=x1,sd=.01)
y = rnorm(20,mean=3+x1+x2)
# coef=lm(y~x1+x2)$coef
coef=lm.ridge(y~x1+x2,lambda=1)$coef
b0=c(b0,coef[1])
b1=c(b1,coef[2])
b2=c(b2,coef[3])
}
plot(1:50,b1,xlab='',ylab='coefficient',type='l',col=3,ylim=c(min(c(b1,b2)),max(c(b1,b2))))
lines(1:50,b2,col=2)
lines(1:50,b0,col=1)
legend('topright',lty=1,c('b0','b1','b2'),col=1:3)

# (Intercept) x1 x2
# 2.582064 39.971344 -38.040040
model=lm.ridge(y~x1+x2,lambda=1)

# x1 x2
# 2.6214998 0.9906773 0.8973912
# ---------------------------------------------------

# ---------------------------------------------------
#      Example of use of lasso regression
# ---------------------------------------------------
library('glmnet')
library(lars)
x = matrix(rnorm(30),10,3)
y = rnorm(10,mean=3+x[,1]+2*x[,3]+0.1*x[,3])
fit1 = lm(y~x)
fit1$coef
# (Intercept)          x1          x2          x3 
# 3.21834508  0.50824017  1.98395606  0.04130131
newx=matrix(rnorm(30),10,3)
newy=rnorm(10,mean=3+newx[,1]+2*newx[,3]+0.1*newx[,3])
yols=predict(fit1,newx=newx)
mean((yols-newy)^2)
# [1] 10.43411
fit2=glmnet(x,y)
coef(fit2,s=1)
# (Intercept)          x1          x2          x3 
# 3.4559250  .  0.5613709  .
ylasso=predict(fit2,newx,s=1)
mean((ylasso-newy)^2)
# [1] 9.888862
fit3=lars(x,y,type="lasso",trace=TRUE)
plot(fit3)
# ---------------------------------------------------

# ======================================== 
# ========== Astronomy data =============
# ======================================== 
data=read.table('./Data/vagc.txt',header=TRUE)
# delete missing data
data=na.omit(data)

datanew=read.table('./Data/vagc2.txt',header=TRUE)

# packages loading
library(lars)
library(MASS)
library(glmnet)
library(pls)
library(FactoMineR)

# modele 1 : 43 ~ 2:42 + 45:50
y=as.numeric(data[,43])
x=data[,c(2:42,45:50)]
dat=data.frame(y,x)

ynew=as.numeric(datanew[,43])
xnew=as.matrix(datanew[,c(2:42,45:50)])
datnew=data.frame(ynew,xnew)

# ============ ridge ==============
model_ridge <- lm.ridge(y~.,data=dat,lambda=1:100)
plot(model_ridge$GCV,xlab='lambda',ylab='GCV',main='Generalized Cross Validation')
which.min(model_ridge$GCV)
# prediction accuracy
ynewhat= scale(xnew, center = model_ridge$xm, scale = model_ridge$scales) %*% model_ridge$coef[, which.min(model_ridge$GCV)] + model_ridge$ym
mean(ynewhat-ynew)^2
# 0.08107475

#============ lasso ==============
model_lasso <- lars(as.matrix(x),y,type="lasso",trace=TRUE,normalize=TRUE)
plot(model_lasso,xvar='df', plottype='coeff')
plot(summary(model_lasso)$Df,summary(model_lasso)$Rss,xlab='Df',ylab='Rss',main='LASSO')
plot(model_lasso,xvar='df', plottype='Cp')
plot(model_lasso, plottype='coeff')
print(model_lasso$Cp-(model_lasso$df-1)) #closer lower Cp to p+1 is for p=8
model_lasso$beta[9,model_lasso$beta[9,]!=0]
# prediction accuracy
ynew_lasso=predict(model_lasso,newx=xnew,s=8,mode="step")
EQM=mean(ynew_lasso$fit-ynew)^2
# 0.08394314

#============= Elastic Net ==============
model_en=cv.glmnet(as.matrix(x),y,alpha=0.5,parallel=TRUE)
plot(model_en)
model_en$lambda.min
# 0.08233859
model_en=glmnet(as.matrix(x),y,alpha=0.5,lambda=model_en$lambda.min)
# prediction accuracy
ynew_en=predict(model_en,newx=xnew)
mean(ynew_en-ynew)^2
# 0.08737518

#=============== lm =============
model_lm=lm(y~.,data=dat)
ynew_lm=predict(model_lm,newdata=data.frame(xnew))
mean(ynew_lm-ynew)^2
# 0.1026039

#=============== PCA =============
pcax=PCA(x)

#=============== PLS =============
model_pls=plsr(y~.,data=dat,scale=TRUE,validation='CV')
plot(RMSEP(model_pls), legendpos = "topright")
plot(model_pls, ncomp = 2, asp = 1, line = TRUE)
plot(model_pls, plottype = "scores", comps = 1:2)

model_pls=plsr(y~.,data=dat,scale=TRUE,ncomp=2)
ynew_lm=predict(model_pls,newdata=data.frame(xnew))
mean(ynew_lm-ynew)^2
# [1] 0.0834256

#=============== PCR =============
model_pcr=pcr(y~.,data=dat,scale=TRUE,validation='CV')
plot(RMSEP(model_pcr), legendpos = "topright")
plot(model_pcr, ncomp = 5, asp = 1, line = TRUE)
plot(model_pcr, plottype = "scores", comps = 1:2)

model_pcr=pcr(y~.,data=dat,scale=TRUE,ncomp=5)
ynew_lm=predict(model_pcr,newdata=data.frame(xnew))
mean(ynew_lm-ynew)^2
# [1] 0.08367541
