#=======================================
# Exercices on kernem methods and dimension reduction for regression
#=======================================

#================================================
# Part one: One-dimensionnal kernel regression -
# influence of the smoothing parameter (bandwidth)
#===============================================

# 1) Simulation of the data

n<-100
x<-runif(n,0,1)
epsilon<-rnorm(n,0,sd=1/9)
y1<-2+3*x+epsilon
y2<-sin(4*x)+epsilon

par(mfrow=c(1,2))
plot(x,y1)
lines(sort(x),2+3*sort(x),col=2)
plot(x,y2)
lines(sort(x),sin(4*sort(x)),col=2)

# 2a) Estimation of the link function via linear model

par(mfrow=c(1,2))
res1<-lm(y1~x)
X<-cbind(rep(1,n),sort(x))
ypred1<-X%*%res1$coef
plot(x,y1)
lines(sort(x),2+3*sort(x),col=2)
lines(sort(x),ypred1,col=3)

res2<-lm(y2~x)
ypred2<-X%*%res2$coef
plot(x,y2)
lines(sort(x),sin(4*sort(x)),col=2)
lines(sort(x),ypred2,col=3)

# 2b) Estimation of the link function via kernel smoothing

res1bis<-ksmooth(x,y1,kernel="normal",bandwidth=0.2)
res2bis<-ksmooth(x,y2,kernel="normal",bandwidth=0.2)

par(mfrow=c(1,2))
plot(x,y1)
lines(sort(x),2+3*sort(x),col=2)
lines(sort(x),ypred1,col=3)
lines(res1bis,col=4)
plot(x,y2)
lines(sort(x),sin(4*sort(x)),col=2)
lines(sort(x),ypred2,col=3)
lines(res2bis,col=4)

#2c) Kernel smoothing using various bandwidths

res1K0.2<-ksmooth(x,y1,kernel="normal",bandwidth=0.2)
res1K0.01<-ksmooth(x,y1,kernel="normal",bandwidth=0.01)
res1K0.5<-ksmooth(x,y1,kernel="normal",bandwidth=0.5)

res2K0.2<-ksmooth(x,y2,kernel="normal",bandwidth=0.2)
res2K0.01<-ksmooth(x,y2,kernel="normal",bandwidth=0.01)
res2K0.5<-ksmooth(x,y2,kernel="normal",bandwidth=0.5)

par(mfrow=c(1,2))
plot(x,y1)
lines(res1K0.01,col=2)
lines(res1K0.2,col=3)
lines(res1K0.5,col=4)
plot(x,y2)
lines(res2K0.01,col=2)
lines(res2K0.2,col=3)
lines(res2K0.5,col=4)

#3) Cross-validation

CV.prog<-function(x,y,hmin=(max(x)-min(x))/20,hmax=(max(x)-min(x))/2,nbh=25){
n<-length(x)
vecth<-seq(from=hmin,to=hmax,length=nbh)
matCV<-cbind(vecth,rep(0,nbh))
for (h in 1:nbh){
ypred<-rep(0,n)
for (i in 1:n){
ypred[i]<-ksmooth(x[-i],y[-i],kernel="normal",band=vecth[h],x.points=x[i])$y
}
matCV[h,2]<-sum((y-as.matrix(ypred,ncol=1))^2)
}
hopt<-matCV[which(matCV[,2]==min(matCV[,2])),1]
list(matCV=matCV,hopt=hopt)
}

par(mfrow=c(1,2))
resCV1<-CV.prog(x,y1)
plot(resCV1$matCV)
abline(v=resCV1$hopt,col=2)

resCV2<-CV.prog(x,y2)
plot(resCV2$matCV)
abline(v=resCV2$hopt,col=2)

res1Khopt<-ksmooth(x,y1,kernel="normal",bandwidth=resCV1$hopt)
res2Khopt<-ksmooth(x,y2,kernel="normal",bandwidth=resCV2$hopt)

par(mfrow=c(1,2))
plot(x,y1)
lines(res1Khopt,col=2)
plot(x,y2)
lines(res2Khopt,col=2)

#====================================
# Part 2: sliced inverse regression
#====================================

n<-100
p<-5
beta<-matrix(c(1,2,-1,-2,0),ncol=1)
X<-matrix(runif(p*n),ncol=p)
index<-X%*%beta
epsilon<-rnorm(n,0,0.5)
y<-(index)^3+epsilon

matYX<-cbind(y,X)
dimnames(matYX)<-list(c(1:n),c("y","x1","x2","x3","x4","x5"))
pairs(matYX)

par(mfrow=c(1,1))
plot(index,y)
u<-seq(min(index),max(index),length=50)
lines(u,u^3,col=2)

library(edrGraphicalTools)

resSIR <- edr(Y=y,X=X,H=5,K=1,method="SIR-I")
plot(resSIR$eigvalEDR,xlab="k",ylab="Eigenvalue")
indexpred1<-X%*%resSIR$matEDR[,1]
indexpred2<-X%*%resSIR$matEDR[,2]
par(mfrow=c(1,2))
plot(indexpred1,y)
plot(indexpred2,y)

round(resSIR$matEDR[,1],digit=3)

par(mfrow=c(1,1))
plot(index,indexpred1)
title(paste("correlation=",round(cor(index,indexpred1),digit=3)))

resCVsir<-CV.prog(indexpred1,y)
plot(resCVsir$matCV)
abline(v=resCVsir$hopt,col=2)
res1Khopt<-ksmooth(indexpred1,y,kernel="normal",bandwidth=resCVsir$hopt)
par(mfrow=c(1,1))
plot(indexpred1,y)
lines(res1Khopt,col=2)


#============================================
# FIN
#============================================



